SystemOrganization addCategory: #'PackageInfo-Extras'!

!ChangeSorter methodsFor: '*packageinfo-extras-changeSet menu' stamp: 'nk 1/20/2004 15:51'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"

	Smalltalk isMorphic ifTrue:
		[aMenu title: 'Change set (shifted)'.
		aMenu addStayUpItemSpecial].

	"CONFLICTS SECTION"
	aMenu add: 'conflicts with other change sets' action: #browseMethodConflicts.
	aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'.
	parent ifNotNil:
		[aMenu add: 'conflicts with change set opposite' action: #methodConflictsWithOtherSide.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in the one on the opposite side of the change sorter.'.

			aMenu add: 'conflicts with category opposite' action: #methodConflictsWithOppositeCategory.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in ANY change set in the category list on the opposite side of this change sorter, other of course than this change set itself.  (Caution -- this could be VERY slow)'].
	aMenu addLine.

	"CHECKS SECTION"
	aMenu add: 'check for slips' action: #lookForSlips.
	aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'.

	aMenu add: 'check for unsent messages' action: #checkForUnsentMessages.
	aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'.

	aMenu add: 'check for uncommented methods' action: #checkForUncommentedMethods.
	aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'.

	aMenu add: 'check for uncommented classes' action: #checkForUncommentedClasses.
	aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'.

	Utilities authorInitialsPerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' action: #checkForAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Utilities authorInitials, '"'.

	aMenu add: 'check for any other authors' action: #checkForAnyAlienAuthorship.
	aMenu balloonTextForLastItem:
'Check this change set for methods any of whose authoring stamps do not start with "', Utilities authorInitials, '"'].

	aMenu add: 'check for uncategorized methods' action: #checkForUnclassifiedMethods.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'.
	aMenu addLine.

	aMenu add: 'inspect change set' action: #inspectChangeSet.
	aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'.

	aMenu add: 'remove empty unnamed change sets' action: #removeEmptyUnnamedChangeSets.
	aMenu balloonTextForLastItem: 
'Removes all the empty change sets with names like Unnamed* or MC<digit>+'.

	aMenu add: 'add all methods to package...' action: #addToPackage.
	aMenu balloonTextForLastItem:
'Prompts for a package to add all the change set''s methods to, then fixes their category names and opens a browser on the changed methods.'.

	aMenu add: 'explode by package' action: #explodeByPackage.
	aMenu balloonTextForLastItem:
'Separates the methods in the selected change set into separate change sets by package ownership'.

	aMenu add: 'update' action: #update.
	aMenu balloonTextForLastItem: 
'Update the display for this change set.  (This is done automatically when you activate this window, so is seldom needed.)'.

	aMenu add: 'go to change set''s project' action: #goToChangeSetsProject.
	aMenu balloonTextForLastItem: 
'If this change set is currently associated with a Project, go to that project right now.'.

	aMenu add: 'promote to top of list' action: #promoteToTopChangeSet.
	aMenu balloonTextForLastItem:
'Make this change set appear first in change-set lists in all change sorters.'.

	aMenu add: 'trim history' action: #trimHistory.
	aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'.

	aMenu add: 'remove contained in class categories...'
		action: #removeContainedInClassCategories.
	aMenu 
		balloonTextForLastItem: ' Drops any changes in given class categories'.
		
	aMenu add: 'clear this change set' action: #clearChangeSet.
	aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'.
	aMenu add: 'expunge uniclasses' action: #expungeUniclasses.
	aMenu balloonTextForLastItem:
'Remove from the change set all memory of uniclasses, e.g. classes added on behalf of etoys, fabrik, etc., whose classnames end with a digit.'.

	aMenu add: 'uninstall this change set' action: #uninstallChangeSet.
	aMenu balloonTextForLastItem: 
'Attempt to uninstall this change set. CAUTION: this may not work completely and is irreversible!!'.

	aMenu addLine.
	aMenu add: 'file into new...' action: #fileIntoNewChangeSet.
	aMenu balloonTextForLastItem: 
'Load a fileout from disk and place its changes into a new change set (seldom needed -- much better to do this from a file-list browser these days.)'.

	aMenu add: 'reorder all change sets' action: #reorderChangeSets.
	aMenu balloonTextForLastItem:
'Applies a standard reordering of all change-sets in the system -- at the bottom will come the sets that come with the release; next will come all the numbered updates; finally, at the top, will come all other change sets'.


	aMenu addLine.

	aMenu add: 'more...' action: #offerUnshiftedChangeSetMenu.
	aMenu balloonTextForLastItem: 
'Takes you back to the primary change-set menu.'.

	^ aMenu! !

!ChangeSorter methodsFor: '*packageinfo-extras-changeSet menu' stamp: 'avi 10/13/2003 15:24'!
addToPackage
	"Prompt for a package name, then fix all of the methods in this change set so they are in the package. Display a message list with the changed messages"
	| packageNames choice |
	packageNames _ PackageOrganizer default packageNames asSortedCollection.
	choice _ PopUpMenu withCaption: 'Add methods to which package?' chooseFrom: packageNames.
	choice isZero ifTrue: [ ^self ].
	myChangeSet browseAfterFixingAllMethodsToBeInPackageNamed: (packageNames at: choice)
! !

!ChangeSorter methodsFor: '*packageinfo-extras-changeSet menu' stamp: 'nk 1/3/2004 17:35'!
removeEmptyUnnamedChangeSets
	self class removeEmptyUnnamedChangeSets.
	self update.! !

!ChangeSorter methodsFor: '*packageinfo-extras-changeSet menu' stamp: 'nk 1/3/2004 17:35'!
explodeByPackage
	myChangeSet explodeByPackage.
	self update.! !

!MessageNames methodsFor: '*packageinfo-extras-selector list' stamp: 'nk 6/24/2003 14:20'!
canResetFilters
	"I can recreate my list, so answer true"
	^true! !

!MessageNames methodsFor: '*packageinfo-extras-selector list' stamp: 'nk 6/24/2003 14:20'!
resetFilters
	"reconstitute my list"
	messageList _ nil.
	self messageList.! !

!ChangedMessageSet methodsFor: '*packageinfo-extras-message list' stamp: 'nk 6/24/2003 14:20'!
canResetFilters
	"I can re-create my list, so answer true"
	^true! !

!ChangedMessageSet methodsFor: '*packageinfo-extras-message list' stamp: 'nk 9/17/2003 16:27'!
addModelItemsToWindowMenu: aMenu
	"Add model-related item to the window menu"

	super addModelItemsToWindowMenu: aMenu. 
	aMenu addLine;
		add: 'remove all from change set' target: self action: #removeAllFromChangeSet;
		add: 'invert' target: self action: #diffFromChangeSet! !

!ChangedMessageSet methodsFor: '*packageinfo-extras-message list' stamp: 'nk 6/24/2003 12:56'!
filterable
	"Answer whether the receiver can be changed by manual deletions"

	^ true! !

!ChangedMessageSet methodsFor: '*packageinfo-extras' stamp: 'nk 9/17/2003 16:33'!
removeAllFromChangeSet
	messageList do: [ :mr | changeSet removeSelectorChanges:  mr methodSymbol class: mr actualClass ]
! !

!ChangedMessageSet methodsFor: '*packageinfo-extras-message list' stamp: 'nk 6/24/2003 12:56'!
growable
	"Answer whether the receiver can be changed by manual additions"

	^ false! !

Object subclass: #PackageChangesService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Extras'!

!PackageChangesService class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 13:50'!
browseCodeFileDifferences: aPackageInfo
	self browseDifferenceForPackage: aPackageInfo againstCodeFile: FileList2 modalFileSelector startingFrom: 0 ! !

!PackageChangesService class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 13:35'!
browseSomeChangesFileDifferences: aPackageInfo
	|full block filteredChanges |
	full _ (ChangeList recentLogOn: FileList2 modalFileSelector startingFrom: 0).
	block _ [:cat :cl | aPackageInfo includesMethodCategory: cat ofClassNamed: cl].
	filteredChanges _ (full changeList select: [:e | e type = #method]) select: [:e | block value: e category value: e methodClassName].
	full changeTo: filteredChanges.
	full removeOlderMethodVersions; removeExistingMethodVersions.
	ChangeList open: full name: 'ba' multiSelect: true! !

!PackageChangesService class methodsFor: 'private' stamp: 'avi 10/11/2003 13:36'!
filterChangesList: changesList forPackage: aPackageInfo
	| filteredChanges |
	changesList
		removeOlderMethodVersions;
		removeExistingMethodVersions.
	filteredChanges := changesList changeList select: 
					[:e | 
					e type = #method and: 
							[aPackageInfo includesMethodCategory: e category
								ofClassNamed: e methodClassName]].
	changesList changeTo: filteredChanges! !

!PackageChangesService class methodsFor: 'services' stamp: 'avi 10/13/2003 15:46'!
unload
	PackageServices unregister: self! !

!PackageChangesService class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 13:45'!
browseChangesFileDifferences: aPackageInfo
	| targetFile startPosition |
	targetFile := SourceFiles at: 2.
	startPosition := Smalltalk lastQuitLogPosition.
	self browseDifferenceForPackage: aPackageInfo againstCodeFile: targetFile
		startingFrom: startPosition! !

!PackageChangesService class methodsFor: 'private' stamp: 'avi 10/11/2003 13:44'!
browseDifferenceForPackage: aPackageInfo againstCodeFile: targetFile startingFrom: startPosition 
	| changesList |
	changesList := ChangeList recentLogOn: targetFile
				startingFrom: startPosition.
	self filterChangesList: changesList forPackage: aPackageInfo.
	ChangeList 
		open: changesList
		name: 'Changes between ' , aPackageInfo packageName , ' and ' 
				, targetFile name
		multiSelect: true! !

!PackageChangesService class methodsFor: 'services' stamp: 'avi 10/13/2003 15:50'!
services
	^ #((browseChangesFileDifferences: 'recover changes')
	     (browseSomeChangesFileDifferences: 'browse changes in file...')
	     (browseCodeFileDifferences: 'browse differences with file...')) collect:
		[:pair |
		SimpleServiceEntry 
			provider: self
			label: pair second
			selector: pair first
			description: pair second]! !

!PackageChangesService class methodsFor: 'services' stamp: 'avi 10/13/2003 15:46'!
initialize
	PackageServices register: self! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 1/23/2004 07:55'!
methodRemovals
	"Answer a collection of MethodReferences"
	^ Array
		streamContents: [:s | changeRecords
				keysAndValuesDo: [:className :classChangeRecord | 
					| realClass | 
					realClass := nil.
					classChangeRecord methodChanges
						keysAndValuesDo: [:selector :methodChangeRecord | 
							| info | 
							info := methodChangeRecord methodInfoFromRemoval.
							info
								ifNotNil: [realClass
										ifNil: [realClass := classChangeRecord realClass].
									realClass
										ifNotNil: [s
												nextPut: (MethodReference class: realClass selector: selector)]]]]]! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 8/26/2003 15:07'!
browseAfterFixingAllMethodsToBeInPackageNamed: aPackageName
	"Fix all of my methods so that they're included in aPackageName. Then browse the changed methods."
	| bad |
	bad _ self fixAllMethodsToBeInPackageNamed: aPackageName.
	bad isEmpty ifTrue: [ ^self inform: 'None!!' ].
	self systemNavigation 
		browseMessageList: bad 
		name: self name, ' methods that have been put in package ', aPackageName	! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 8/26/2003 15:08'!
browseMethodsNotInCoreOfPackage: packageName 
	"Open a message list browser on the new and changed methods in the receiver which are not in the core classes of the PackageInfo named 'packageName'"

	| aList |
	aList := self methodsNotInCoreOfPackageNamed: packageName.
	aList isEmpty ifTrue: [^self inform: 'None!!'].
	self systemNavigation  browseMessageList: aList
		name: self name , ' methods that are not in core classes of package ' 
				, packageName! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 1/23/2004 07:12'!
probablePackage
	| ccl cml possible |
	ccl _ self changedClasses collect: [ :ea | ea category ] thenSelect: [ :ea | ea notNil ].
	possible _ PackageInfo allPackages select: [ :pkg | ccl anySatisfy: [ :c | pkg includesSystemCategory: c ]].
	possible isEmpty ifTrue: [ ^nil ].
	possible size = 1 ifTrue: [ ^possible first ].
	cml _ self changedMessageListAugmented.
	^possible detectMax: [ :pkg | cml count: [ :mr | pkg includesMethodReference: mr ]]! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 8/26/2003 15:08'!
browseMethodsNotInPackage: packageName 
	"Open a message list browser on the new and changed methods in the receiver which are not in the PackageInfo named 'packageName'"

	| aList |
	aList := self methodsNotInPackageNamed: packageName.
	aList isEmpty ifTrue: [^self inform: 'None!!'].
	self systemNavigation  browseMessageList: aList
		name: self name , ' methods that are not in package ' , packageName! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 1/23/2004 08:06'!
extensionMethodRemovals
	"Answer a collection of MethodReferences"
	^ Array
		streamContents: [:s | changeRecords
				keysAndValuesDo: [:className :classChangeRecord | 
					| realClass | 
					realClass := nil.
					classChangeRecord methodChanges
						keysAndValuesDo: [:selector :methodChangeRecord | 
							| info | 
							info := methodChangeRecord methodInfoFromRemoval.
							(info notNil
									and: [info second notNil
											and: [info second first = $*]])
								ifTrue: [realClass
										ifNil: [realClass := classChangeRecord realClass].
									realClass
										ifNotNil: [| splitInfo splitCat | 
											splitInfo := info second findTokens: '*-'.
											splitCat := realClass theNonMetaClass category findTokens: '-'.
											(splitInfo first sameAs: splitCat first)
												ifFalse: [s
														nextPut: (MethodReference class: realClass selector: selector)]]]]]]! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 1/23/2004 07:54'!
browseAllExtensionMethodRemovals
	"Browse all the extension methods that I removed."
	| removals pkg |
	removals _ self extensionMethodRemovals.
	removals ifEmpty: [ ^self ].
	pkg _ (self probablePackage ifNotNilDo: [ :p | ' (package ', p packageName, ')' ]) ifNil: [ '' ].
	self systemNavigation 
		browseMessageList: self methodRemovals 
		name: self name, pkg, ' extension method removals'! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 9/9/2003 16:17'!
explodeByPackage
	"Make 0 or more new change sets with subsets of my changes that would belong to various projects."
	| cml allNew |
	cml _ self changedMessageList.
	allNew _ Dictionary new.
	PackageInfo allPackages do: [ :pkg | | pkgList newCSName newCS |
		pkgList _ cml select: [ :mr | pkg includesMethodReference: mr ].
		newCSName _ (self name, '-', pkg packageName).
		newCS _ ChangeSorter existingOrNewChangeSetNamed: newCSName.
		allNew at: pkg put: newCS.
		pkgList do: [ :mr | | sel |
			sel _ mr methodSymbol.
			(MessageSet isPseudoSelector: sel) ifFalse: [
			newCS absorbMethod: sel class: mr actualClass from: self ]].
		cml _ cml copyWithoutAll: pkgList.
	].
	allNew keysAndValuesDo: [ :pkg :cs |
		changeRecords keys do: [ :clsName | | realName parts |
			parts _ clsName asString findTokens: ' '.
			realName _ parts first asSymbol.
			Smalltalk at: realName ifPresentAndInMemory: [ :cls |
				(pkg includesClass: cls) ifTrue: [
					cs absorbClass: realName from: self.
					self removeClassChanges: realName.
					cs absorbClass: (cls class) name from: self.
					self removeClassChanges: (cls class) name ]]]].
	self expungeEmptyClassChangeEntries.
	(allNew select: [ :cs | cs isEmpty ]) do: [ :cs | ChangeSorter removeChangeSet: cs ]! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 1/23/2004 07:16'!
methodsNotInProbablePackage
	"Answer a collection of MethodReferences for my method changes which aren't contained in my probable package"

	| pi |
	pi := self probablePackage.
	pi ifNil: [ ^#() ].
	^self methodsNotInPackageNamed: pi packageName.! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 6/19/2003 19:30'!
methodsNotInPackageNamed: aPackageName 
	"Answer a collection of MethodReferences for my method changes which aren't contained in aPackageName"

	| pi |
	pi := PackageInfo named: aPackageName.
	^self messageListForChangesWhich: 
			[:cl :ms | 
			cl notNil 
				and: [ms ~= 'Comment' and: [(pi includesMethod: ms ofClass: cl) not]]]
		ifNone: [#()]! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 6/19/2003 19:31'!
methodsNotInCoreOfPackageNamed: aPackageName 
	"Answer a collection of MethodReferences for my method changes which aren't contained in aPackageName"

	| pi |
	pi := PackageInfo named: aPackageName.
	^self messageListForChangesWhich: 
			[:cl :ms | 
			cl notNil and: [ms ~= 'Comment' and: [(pi includesClass: cl) not]]]
		ifNone: [#()]! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 6/20/2003 16:38'!
fixAllMethodsToBeInPackageNamed: aPackageName 
	"For all of my method changes and additions, make sure that the 
	category prefixes allow their inclusion in aPackageName. Answer the 
	fixed methods."
	| pi bad prefix redundant |
	pi _ PackageInfo named: aPackageName.

	bad _ self methodsNotInPackageNamed: aPackageName.
	redundant _ self methodsRedundantlyCategorizedForPackageNamed: aPackageName.
	bad isEmpty
		and: [redundant isEmpty
				ifTrue: [^ bad]].

	prefix _ pi methodCategoryPrefix allButFirst asLowercase.
	bad do: [:mr | PackageInfo reCategorizeForeignMethod: mr forPackageNamed: prefix ].

	redundant do: [:mr | PackageInfo reCategorizeOwnMethod: mr forPackageNamed: prefix ].
	^ bad , redundant! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 1/23/2004 08:24'!
extensionMethodRemovalRecords
	"Answer a collection of MethodReferences"
	^ Array
		streamContents: [:s | changeRecords
				keysAndValuesDo: [:className :classChangeRecord | 
					| realClass | 
					realClass := nil.
					classChangeRecord methodChanges
						keysAndValuesDo: [:selector :methodChangeRecord | 
							| info | 
							info := methodChangeRecord methodInfoFromRemoval.
							(info notNil
									and: [info second notNil
											and: [info second first = $*]])
								ifTrue: [realClass
										ifNil: [realClass := classChangeRecord realClass].
									realClass
										ifNotNil: [| splitInfo splitCat | 
											splitInfo := info second findTokens: '*-'.
											splitCat := realClass theNonMetaClass category findTokens: '-'.
											(splitInfo first sameAs: splitCat first)
												ifFalse: [s
														nextPut: (methodChangeRecord)]]]]]]! !

!ChangeSet methodsFor: '*packageinfo-extras-packages' stamp: 'nk 6/19/2003 20:06'!
methodsRedundantlyCategorizedForPackageNamed: aPackageName 
	"Answer a collection of MethodReferences for my method changes which have a prefix for aPackageName but don't need one."

	| pi org cat |
	pi := PackageInfo named: aPackageName.
	^self messageListForChangesWhich: 
			[:cl :ms | 
			cl notNil and: 
					[ms ~= 'Comment' and: 
							[(pi includesClass: cl) and: 
									[org := cl organization.
									cat _ org categoryOfElement: ms.
									cat notNil and: [ pi isYourClassExtension: cat ]]]]]
		ifNone: [#()]! !

!PackageInfo class methodsFor: '*packageinfo-extras-enumeration' stamp: 'nk 6/20/2003 16:36'!
reCategorizeForeignMethod: aMethodReference forPackageNamed: packageName
	"Answer a (possibly modified) category name for the given method reference"
	| org oldCat newCat oldPrefix tokens | 
	org _ aMethodReference actualClass organization.
	oldCat _ (org categoryOfElement: aMethodReference methodSymbol) ifNil: [ '' ].
	oldCat _ oldCat asString.
	({ 'as yet unclassified'. 'all' } includes: oldCat) ifTrue: [ oldCat _ '' ].
	tokens _ oldCat findTokens: '*-' keep: '*'.

	"Strip off any old prefixes"
	((tokens at: 1 ifAbsent: [ '' ]) = '*') ifTrue: [
		[ ((tokens at: 1 ifAbsent: [ '' ]) = '*') ]
			whileTrue: [ tokens removeFirst ].
		oldPrefix _ tokens removeFirst asLowercase.
		[ (tokens at: 1 ifAbsent: [ '' ]) asLowercase = oldPrefix ]
			whileTrue: [ tokens removeFirst ].
	].

	newCat _ String streamContents: [ :s |
		s nextPut: $*; nextPutAll: packageName asLowercase.
		tokens do: [ :tok | s nextPut: $-; nextPutAll: tok ]
	].

	org
		classify: aMethodReference methodSymbol
		under: newCat! !

!PackageInfo class methodsFor: '*packageinfo-extras-enumeration' stamp: 'nk 6/20/2003 16:34'!
reCategorizeOwnMethod: aMethodReference forPackageNamed: packageName
	"Answer a (possibly modified) category name for the given method reference"
	| org oldCat newCat oldPrefix tokens | 
	org _ aMethodReference actualClass organization.
	oldCat _ (org categoryOfElement: aMethodReference methodSymbol) ifNil: [ '' ].
	oldCat _ oldCat asString.
	({ 'as yet unclassified'. 'all' } includes: oldCat) ifTrue: [ oldCat _ '' ].
	tokens _ oldCat findTokens: '*-' keep: '*'.

	"Strip off any old prefixes"
	((tokens at: 1 ifAbsent: [ '' ]) = '*') ifTrue: [
		[ ((tokens at: 1 ifAbsent: [ '' ]) = '*') ]
			whileTrue: [ tokens removeFirst ].
		oldPrefix _ tokens removeFirst asLowercase.
		[ (tokens at: 1 ifAbsent: [ '' ]) asLowercase = oldPrefix ]
			whileTrue: [ tokens removeFirst ].
	].

	newCat _ String streamContents: [ :s |
		tokens do: [ :tok | s nextPutAll: tok ]
			separatedBy: [ s nextPut: $- ].
	].

	org
		classify: aMethodReference methodSymbol
		under: newCat! !

!MessageSet methodsFor: '*packageinfo-extras-filtering' stamp: 'nk 6/24/2003 12:46'!
filterMessageList
	"Allow the user to refine the list of messages."

	| aMenu evt |
	Smalltalk isMorphic ifFalse: [^ self inform: 'sorry, morphic only at this time.'].
	messageList size <= 1 ifTrue: [^ self inform: 'this is not a propitious filtering situation'].

	"would like to get the evt coming in but thwarted by the setInvokingView: circumlocution"
	evt _ self currentWorld activeHand lastEvent.
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Filter by only showing...'.
	aMenu addStayUpItem.

	aMenu addList: #(
		('unsent messages'						filterToUnsentMessages		'filter to show only messages that have no senders')
		-
		('messages that send...'					filterToSendersOf			'filter to show only messages that send a selector I specify')
		('messages that do not send...'			filterToNotSendersOf		'filter to show only messages that do not send a selector I specify')
		-
		('messages whose selector is...'			filterToImplementorsOf		'filter to show only messages with a given selector I specify')
		('messages whose selector is NOT...'		filterToNotImplementorsOf	'filter to show only messages whose selector is NOT a selector I specify')
		-
		('messages in current change set'		filterToCurrentChangeSet	'filter to show only messages that are in the current change set')
		('messages not in current change set'	filterToNotCurrentChangeSet	'filter to show only messages that are not in the current change set')
		-
		('messages in any change set'			filterToAnyChangeSet		'filter to show only messages that occur in at least one change set')
		('messages not in any change set'		filterToNotAnyChangeSet		'filter to show only messages that do not occur in any change set in the system')
		-
		('messages in package...'				filterToMethodsInPackage	'filter to show only messages in a package that I specify')
		('messages not in package...'			filterToNotMethodsInPackage	'filter to show only messages NOT in a package that I specify')
		('messages in any package'				filterToMethodsInAnyPackage	'filter to show only messages in some package')
		('messages not in any package'			filterToMethodsNotInAnyPackage	'filter to show only messages in NO package')
		-
		('messages authored by me'				filterToCurrentAuthor		'filter to show only messages whose authoring stamp has my initials')
		('messages not authored by me'			filterToNotCurrentAuthor	'filter to show only messages whose authoring stamp does not have my initials')
		-
		('messages logged in .changes file'		filterToMessagesInChangesFile	'filter to show only messages whose latest source code is logged in the .changes file')
		('messages only in .sources file'			filterToMessagesInSourcesFile	'filter to show only messages whose latest source code is logged in the .sources file')
		-
		('messages with prior versions'			filterToMessagesWithPriorVersions	'filter to show only messages that have at least one prior version')
		('messages without prior versions'		filterToMessagesWithoutPriorVersions	'filter to show only messages that have no prior versions')
		-
		('uncommented messages' filterToUncommentedMethods 'filter to show only messages that do not have comments at the beginning')
		('commented messages' filterToCommentedMethods 'fileter to show only messages that have comments at the beginning')
		-
		('messages in hardened classes'			filterToMessagesWithHardenedClasses	'filter to show only messages of established classes (as opposed to Uniclasses such as Player23)')
		-
		('messages that...'						filterToMessagesThat			'let me type in a block taking a class and a selector, which will specify yea or nay concerning which elements should remain in the list')
			).

	aMenu popUpEvent: evt hand lastEvent in: evt hand world.! !

!MessageSet methodsFor: '*packageinfo-extras-filtering' stamp: 'nk 6/24/2003 14:21'!
filterToMethodsInPackage
	"Display only methods that are represented by a specific PackageInfo"
	| packageNames choice pi |
	packageNames _ PackageInfo allPackageNames asSortedCollection.
	choice _ PopUpMenu withCaption: 'Which package?' chooseFrom: packageNames.
	choice isZero ifTrue: [ ^self ].
	choice _ packageNames at: choice.
	pi := PackageInfo named: choice.
	self setFilteredList: (self messageList select: [ :mr |
		pi includesMethodReference: mr ]
		)! !

!MessageSet methodsFor: '*packageinfo-extras-filtering' stamp: 'nk 6/24/2003 14:22'!
resetFilters
	"reconstitute my list without filters"
	self reformulateList! !

!MessageSet methodsFor: '*packageinfo-extras-message list' stamp: 'nk 6/24/2003 12:55'!
addExtraShiftedItemsTo: aMenu 
	"The shifted selector-list menu is being built. Add items specific to 
	MessageSet "
	self filterable ifTrue: [
		aMenu addLine.
		aMenu add: 'remove from this browser' action: #removeMessageFromBrowser.
		aMenu add: 'filter message list...'  action: #filterMessageList.
		self canResetFilters ifTrue: [
			aMenu add: 'reset filters' action: #resetFilters]].
	self growable
		ifTrue: [aMenu add: 'add to message list...' action: #augmentMessageList].
	aMenu add: 'sort by date' action: #sortByDate! !

!MessageSet methodsFor: '*packageinfo-extras-private' stamp: 'nk 6/29/2003 17:59'!
initializeMessageList: anArray
	| s oldSelection |
	
	oldSelection _ self selection.

	messageList _ OrderedCollection new.
	anArray do: [ :each |
		MessageSet 
			parse: each  
			toClassAndSelector: [ :class :sel |
				class ifNotNil: [
					s _ class name , ' ' , sel , ' {' , 
						((class organization categoryOfElement: sel) ifNil: ['']) , '}'.
					messageList add: (
						MethodReference new
							setClass: class  
							methodSymbol: sel 
							stringVersion: s
					)
				]
			]
	].

	self messageListIndex: (messageList indexOf: oldSelection).
! !

!MessageSet methodsFor: '*packageinfo-extras-private' stamp: 'nk 6/29/2003 17:59'!
selection
	"Answer the item in the list that is currently selected, or nil if no selection is present"

	messageList ifNil: [ ^nil ].
	^ messageList at: messageListIndex ifAbsent: [nil]! !

!MessageSet methodsFor: '*packageinfo-extras-filtering' stamp: 'nk 6/24/2003 14:21'!
canResetFilters
	"Answer true if I can reconstitute my list"
	^false! !

!MessageSet methodsFor: '*packageinfo-extras-filtering' stamp: 'nk 6/24/2003 14:21'!
filterToMethodsInAnyPackage
	"Display only methods that are represented by a PackageInfo"
	| packages |
	packages _ PackageInfo allPackages.
	self setFilteredList: (self messageList select: [ :mr |
			packages anySatisfy: [ :pi | pi includesMethodReference: mr ]
		])! !

!MessageSet methodsFor: '*packageinfo-extras-filtering' stamp: 'nk 6/24/2003 14:22'!
filterToNotMethodsInPackage
	"Display only methods that are not represented by a specific PackageInfo"
	| packageNames choice pi |
	packageNames _ PackageInfo allPackageNames asSortedCollection.
	choice _ PopUpMenu withCaption: 'Which package?' chooseFrom: packageNames.
	choice isZero ifTrue: [ ^self ].
	choice _ packageNames at: choice.
	pi := PackageInfo named: choice.
self setFilteredList: (self messageList select: [ :mr |
		(pi includesMethodReference: mr) not ]
		)! !

MessageSet subclass: #KeyedMessageSet
	instanceVariableNames: 'keys'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Extras'!

!KeyedMessageSet commentStamp: '<historical>' prior: 0!
(PackageInfo named: 'PackageInfo-Kernel') browseExternalReferences!

!KeyedMessageSet class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 14:39'!
unload
	PackageServices unregister: self! !

!KeyedMessageSet methodsFor: 'as yet unclassified' stamp: 'ab 11/18/2002 01:17'!
keyedMethods: aCollection
	messageList _ aCollection collect:
						[:ea | |ref|
						ref _ ea key.
						ref stringVersion: (ref stringVersion, ' ---> ', ea value printString).
						ref].
	keys _ aCollection collect: [:ea | ea value].! !

!KeyedMessageSet class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 14:04'!
browseExternalReferences: aPackageInfo
	MessageSet
		open: (KeyedMessageSet new keyedMethods:
					aPackageInfo externalCallers asArray, aPackageInfo externalUsers asArray)
		name: aPackageInfo packageName! !

!KeyedMessageSet class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 14:39'!
services
	^ Array with: self browseService! !

!KeyedMessageSet class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 14:39'!
browseService
	^ SimpleServiceEntry 
		provider: self
		label: 'browse external references'
		selector: #browseExternalReferences:
		description: 'open a message set on any methods outside this package that depend upon it'! !

!KeyedMessageSet methodsFor: 'as yet unclassified' stamp: 'ab 11/14/2002 19:03'!
messageListIndex: aNumber
	super messageListIndex: aNumber.
	self changed: #autoSelect ! !

!KeyedMessageSet class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 14:39'!
initialize
	PackageServices register: self! !

!KeyedMessageSet methodsFor: 'as yet unclassified' stamp: 'ab 11/14/2002 19:02'!
autoSelectString
	^ keys at: messageListIndex ifAbsent: ['']! !

!MessageSet methodsFor: '*packageinfo-extras-filtering' stamp: 'nk 6/24/2003 14:22'!
filterToMethodsNotInAnyPackage
	"Display only methods that are represented by no PackageInfo"
	| packages |
	packages _ PackageInfo allPackages.
	self setFilteredList: (self messageList select: [ :mr |
			packages noneSatisfy: [ :pi | pi includesMethodReference: mr ]
		])! !

!MessageSet methodsFor: '*packageinfo-extras-message list' stamp: 'nk 6/24/2003 12:49'!
filterable
	"Answer whether the receiver is capable of growing/shrinking dynamically"

	^ true! !

!RecentMessageSet methodsFor: '*packageinfo-extras-update' stamp: 'nk 6/24/2003 12:50'!
filterable
	"Answer whether the receiver can be changed by manual additions & deletions"

	^ false! !
PackageChangesService initialize!
KeyedMessageSet initialize!
